local oldInteract = FISHING_MANAGER.StartInteraction
local isConfirmed = false
local isHidden = false
local settings = {}
local start

local function stealingAllowed()
	return isHidden or isConfirmed
end

local function hasBound()
	local layerIndex, categoryIndex, actionIndex = GetActionIndicesFromName("CONFIRM_STEAL")
	local bind = GetActionBindingInfo(layerIndex, categoryIndex, actionIndex, 1)
	if bind ~= 0 then
		return true
	end
	bind = GetActionBindingInfo(layerIndex, categoryIndex, actionIndex, 2)
	return bind ~= 0
end

local stealInfoControl = CreateControlFromVirtual("StealInfo", ZO_ReticleContainerInteract, "ZO_KeybindButton")
stealInfoControl:SetAnchor(TOPLEFT, ZO_ReticleContainerInteractKeybindButton, BOTTOMLEFT, 0, 0)
stealInfoControl:SetKeybind("CONFIRM_STEAL", false)

local function setInfoText()
	if hasBound() then
		stealInfoControl:SetText(zo_strformat(SI_GAME_CAMERA_TARGET, "or stealth to allow stealing"))
	else
		stealInfoControl:SetText(zo_strformat(SI_GAME_CAMERA_TARGET, "stealth to allow stealing"))
	end
end

local function updateColor()
	if stealingAllowed() then
		stealInfoControl:SetNormalTextColor(ZO_SUCCEEDED_TEXT)
	else
		stealInfoControl:SetNormalTextColor(ZO_NORMAL_TEXT)
	end
end

-- interaction is cancelled, if this function returns true
FISHING_MANAGER.StartInteraction = function(...)
	if settings.delay < 0 then
		if not start then
			start = GetFrameTimeMilliseconds()
			d("Start setting doubletap timeframe.")
		else
			settings.delay = zo_round((GetFrameTimeMilliseconds() - start) * 1.1) -- 10% increase
			d("Doubletap timeframe set to "..settings.delay.." milliseconds.")
		end
		return
	end
	local isOwned = (not stealingAllowed()) and select(4, GetGameCameraInteractableActionInfo())
	if isOwned then
		ConfirmStealing(true)
		zo_callLater(function() ConfirmStealing(false) end, settings.delay)
		return true
	else
		return oldInteract(...)
	end
end

function ConfirmStealing(confirmed)
	isConfirmed = confirmed
	updateColor()
end

ZO_CreateStringId("SI_BINDING_NAME_CONFIRM_STEAL", "Confirm Stealing")	

local stateChange = RETICLE.OnStealthStateChanged
RETICLE.OnStealthStateChanged = function(self, stealthState)
	isHidden = self.stealthIcon.hiddenStates[stealthState]
	updateColor()
	return stateChange(self, stealthState)
end

local updateInteractText = RETICLE.UpdateInteractText
RETICLE.UpdateInteractText = function(self, ...)
	stealInfoControl:SetHidden(not select(4, GetGameCameraInteractableActionInfo()))
	setInfoText()
	return updateInteractText(self, ...)
end

local function SetTimeframe(delay)
	if delay == "config" then
		d("Doubetap the Interaction-key to set the timeframe.")
		settings.delay = -1
		return
	end
	delay = tonumber(delay)
	if not delay or delay < 0 then
		d("/stealtimeframe x")
		d("Where x is the doubletap timeframe in miliseconds.")
		d("x = 0 disables doubletap as confirmation.")
		d("Type:")
		d("/stealtimeframe config")
		d("to set the timeframe by doubletapping the Interaction-key.")
		return
	end
	settings.delay = delay
	d("Doubletap timeframe set to "..settings.delay.." milliseconds.")
end

local function OnAddonLoaded( _, addon )
	if addon ~= "NoAccidentalStealing" then
		return
	end
	settings = ZO_SavedVars:New("NAS_SavedVariables", 1, "settings", {delay = 0})
	
	SLASH_COMMANDS["/stealtimeframe"] = SetTimeframe
end

EVENT_MANAGER:RegisterForEvent("NoAccidentalStealing", EVENT_ADD_ON_LOADED , OnAddonLoaded)